#include "Entity.h"

using namespace std;

//constructeur
Entity::Entity(vector<double> const& traitPhenotype, double const& traitPosition, double const& timeNow,
               Geometry const& geometrySpace, vector<double> const& parameters,
               double const& probaMutation):
                   m_traitPhenotype(traitPhenotype), m_traitPosition(traitPosition), m_timePosition(timeNow),
                   m_geometrySpace(geometrySpace), m_parameters(parameters),
                   m_probaMutation(probaMutation)
{

}


//destructeur
Entity::~Entity()
{

}



//retourne la geometrie associe a l'individu, a priori, tous les individus ont la meme geometrie
Geometry Entity::getGeometry() const
{
    return m_geometrySpace;
}


//change la position de l'individu, a n'utiliser que pour l'initialisation
void Entity::changePosition(double const& newPosition)
{
    m_traitPosition = newPosition;
}


//retourne la probabilite de mutation associe a l'individu
double Entity::getProbaMutation() const
{
    if (m_probaMutation > 1 && m_probaMutation < 0)
    {
    cout << "problem with mutation probability" << endl;
    assert(false);
    }
    return m_probaMutation;
}

//retourne le premier trait de l'individu
double Entity::getFirstTraitPhenotype() const
{
    return m_traitPhenotype[0];
}


//calcule le coefficient de diffusion de l'Entity a partir du fichier rateMove
double Entity::coefDiffusionEntity() const
{
    return coefDiffusion(m_traitPosition, m_traitPhenotype, m_parameters);
}


//met a jour la position de l'individu et son temps enregistre.
// a partir du brownien defini dans le fichier rateMove
void Entity::updatePosition(double const& timeNow)
{
    m_traitPosition = liveShifting(timeNow - m_timePosition, m_geometrySpace,
                                   m_traitPosition, m_traitPhenotype, m_parameters);
    m_timePosition = timeNow;
}


//calcul du taux de naissance de l'individu au temps timeNow
//a partir du fichier rateMove
double Entity::birthRateEntity(double const& timeNow)
{
    updatePosition(timeNow);
    return birthRate(m_traitPosition, m_traitPhenotype, m_parameters);
}


//calcul de le noyau d'interaction pour un taux positif exerce par competitor sur notre individu au temps timeNow
//a partir du fichier rateMove
double Entity::positiveInteraction(double const& timeNow, Entity partner)
{
    updatePosition(timeNow);
    partner.updatePosition(timeNow);
    return positiveInteractionKernel(m_traitPosition, m_traitPhenotype, m_parameters,
                             partner.m_traitPosition, partner.m_traitPhenotype, partner.m_parameters);
}


//calcul du taux de mort de l'individu au temps timeNow
//a partir du fichier rateMove
double Entity::deathRateEntity(double const& timeNow)
{
    updatePosition(timeNow);
    return birthRate(m_traitPosition, m_traitPhenotype, m_parameters);
}


//calcul de la competition exerce par competitor sur notre individu au temps timeNow
//a partir du fichier rateMove
double Entity::competition(double const& timeNow, Entity competitor)
{
    updatePosition(timeNow);
    competitor.updatePosition(timeNow);
    return competitionKernel(m_traitPosition, m_traitPhenotype, m_parameters,
                      competitor.m_traitPosition, competitor.m_traitPhenotype, competitor.m_parameters);
}

//calcul de la competition exerce par competitor sur notre individu au temps timeNow
//a partir du fichier rateMove
double Entity::competitionMut(double const& timeNow, Entity competitor)
{
    updatePosition(timeNow);
    competitor.updatePosition(timeNow);
    return competitionMutKernel(m_traitPosition, m_traitPhenotype, m_parameters,
                      competitor.m_traitPosition, competitor.m_traitPhenotype, competitor.m_parameters);
}


// calcul d'un nouveau mutant avec un nouveau trait
Entity Entity::mutant(double const& timeNow)
{
    updatePosition(timeNow);
    vector<double> newTrait;
    newTrait = mutantTraitPhenotype(m_traitPhenotype, m_traitPosition, m_parameters, m_geometrySpace);
    //le nouveau ne mutant herite de la position actuelle du parent, des parametres et
    //min/maxPosition qui sont des parametres generaux, lies a son espece!
    Entity newMutant(newTrait, m_traitPosition, timeNow, m_geometrySpace,
                     m_parameters, m_probaMutation);
    return newMutant;
}


//calcul d'une nouvelle position a la naissance
Entity Entity::mutantShiftBirth(double const& timeNow)
{
    updatePosition(timeNow);
    double newPosition;
    newPosition = birthShifting(m_traitPosition, m_traitPhenotype, m_parameters, m_geometrySpace);
    Entity newMutant(m_traitPhenotype, newPosition, timeNow, m_geometrySpace,
                     m_parameters, m_probaMutation);
    return newMutant;
}


//ecrit sur une ligne du fichier namefileOut, les traits phenotypiques, suivis de la
//position et du temps
void Entity::writeEntity(string nameFileOut, double const& timeNow)
{
    ofstream flux(nameFileOut.c_str(), ios::app);
    updatePosition(timeNow);
    int nbrTraitPhenotype(m_traitPhenotype.size());
    for (int k(0) ; k < abs(nbrTraitPhenotype) ; k++)
    {
        flux << m_traitPhenotype[k] << " , ";
    }
    flux << m_traitPosition << " , " << m_timePosition << " , " << m_parameters[0] << endl;
}


//donne les indices du mutant sur la "carte" 2D traitPhenotypique x tr
void Entity::defineUplet(double const& timeNow, double const& sizeIntervalPhenotype,
                         double const& sizeIntervalPosition, int &index1, int &index2)
{
    updatePosition(timeNow);
    index1 = (int) ((m_geometrySpace.distanceMinPhenotype(m_traitPhenotype,0))/sizeIntervalPhenotype);
    index2 = (int) ((m_geometrySpace.distanceMinPosition(m_traitPosition,0))/sizeIntervalPosition);
}


int Entity::getType() const
{
    return m_parameters[0];
}

bool Entity::testPosition(double const& timeNow, double const& testedPosition)
{
    updatePosition(timeNow);
    return (m_traitPosition > testedPosition);
}
